function s = initialize(s);

% initialization routine for measure matching algorithm
%
% author : Joan Glaunès - joan.glaunes@free.fr

% load structure in workspace
loadstruct('s');
clear s

% sort points by groups
ng = max(grpx);
[grpx,gindx] = sort(grpx);
x = x(:,gindx);
[grpy,gindy] = sort(grpy);
y = y(:,gindy);

% compute first index of each group
for g = 1:ng
    lgx(g) = length(find(grpx==g));
    lgy(g) = length(find(grpy==g));
end
gtabx = [1,1+cumsum(lgx)];
gtaby = [1,1+cumsum(lgy)];

% Compute weights for each group
if [length(wx),length(wy)]==[ng,ng]
    for g = 1:ng
        wx = [wx,ones(1,lgx(g))*wx(g)/lgx(g)];
        wy = [wy,ones(1,lgy(g))*wy(g)/lgy(g)];
    end
    wx = wx(ng+1:end);
    wy = wy(ng+1:end);
else
    wx = wx(gindx);
    wy = wy(gindy);
end

% normalize weights
if equalweights
    wx=wx/sum(wx);
    wy=wy/sum(wy);
end

clear g lgx lgy

% prior affine transformation
xrig = transmatrix * x + repmat(transvector,1,size(x,2));

% auto scaling
sclsz = mean(max([x,y]')-min([x,y]'));
if strcmp(sigmaV,'auto')
    sigmaV = 2*sclsz*(T:-1:1)/T;
end
if strcmp(sigmaI,'auto')
    sigmaI = .1*sclsz;
end
if strcmp(gammaR,'auto')
    gammaR = (.001/sclsz)^2;
end
if strcmp(optim_stepsize,'auto')
   optim_stepsize = 1e-9/gammaR;
end
clear sclsz

if length(sigmaV)==1
    sigmaV = sigmaV*ones(1,T);
end
sigmaV2 = sigmaV.^2;
sigmaI2 = sigmaI^2;
tau = 1/(T-1);

% squared norm of target measure
sqnormy = matching([],[],[],[],zeros(1,ng+1),y(1,:),y(2,:),y(3,:),wy,gtaby,sigmaI2);

% reset trajectories and momentum
if resetmode
    X1 = repmat(xrig(1,:),T,1)';
    X2 = repmat(xrig(2,:),T,1)';
    X3 = repmat(xrig(3,:),T,1)';
    mom = zeros(nx,T-1,3);
    clear J   % functional values
end

% initialize functional
J(1) = eval(['functional(mom,',functional,')']);

% build optimization options structure
optvars = who('optim_*');
for v = 1:length(optvars)
    var = optvars{v};
    eval([regexprep(var,'optim_','optim.'),' = ',var,';']);
end
clear optvars var v  

% save structure
vars = who;
vars = who;
savestruct('s',vars)
